package xyz.tbvns.Models;

import com.badlogic.gdx.graphics.g3d.Model;
import com.badlogic.gdx.graphics.g3d.ModelInstance;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import xyz.tbvns.LevelInfo;

@Data
@NoArgsConstructor
public class GDInstance {
    public GDInstance(ModelInstance instance, String name, String id) {
        this.instance = instance;
        this.name = name;
        this.id = id;
        this.animation = false;
    }

    public GDInstance(ModelInstance instance, int fps, int frameSkip, double start, String name, String id, Animation animation) {
        this.instance = instance;
        this.fps = fps;
        this.frameSkip = frameSkip;
        this.start = start;
        this.name = name;
        this.id = id;
        this.animation = true;
        this.animationObject = animation;
        this.expanded = false;
    }

    public void updateFrame(double percent) {
        instance = new ModelInstance(getFrameOnPercent(percent), instance.transform);
        instance.userData = this;
    }

    public Model getFrameOnPercent(double percent) {
        double usablePercent = percent - (start / 100);
        if (usablePercent > 0) {
            float max = LevelInfo.getLevel().getMaxPos();
            double pos = max * usablePercent;
            double speed = LevelInfo.getLevel().getSpeedAt((float) percent);

            double distancePerFrame = (speed / fps) * 10;
            int frame = (int) (pos / distancePerFrame);

            System.out.println(pos + " - " + frame);

            // Ensure the frame is within bounds
            if (frame < animationObject.getModelFrames().size()) {
                return animationObject.getModelFrames().get(frame);
            }
            return animationObject.getModelFrames().getLast();
        }
        return animationObject.getModelFrames().getFirst();
    }


    private ModelInstance instance;
    private boolean animation;
    private double start;
    private int current;
    private int fps, frameSkip;
    //TODO: add keyframes
    private String name, id;
    private boolean expanded;
    private Animation animationObject;
}
